package navigator

import (
	"encoding/json"
	"fmt"
	"git.eve.moe/jackyyf/navigator/ipgeo"
	"git.eve.moe/jackyyf/navigator/mapping"
	"git.eve.moe/jackyyf/navigator/utils"
	"log"
	"net"
	"net/http"
	"strings"
)

const (
	errIPv4Only = "Navigator works for valid IPv4 only :)"
)

func init() {
	http.HandleFunc("/healthz", func(resp http.ResponseWriter, req *http.Request) {
		resp.WriteHeader(200)
		resp.Write([]byte("ok"))
	})

	http.HandleFunc("/info", func(resp http.ResponseWriter, req *http.Request) {
		var host string
		if argIp := req.FormValue("ip"); argIp != "" {
			host = argIp
			if net.ParseIP(host).To4() == nil {
				utils.ResponseWithError(resp, http.StatusPreconditionFailed, errIPv4Only)
				return
			}
		} else {
			host = utils.GetRemoteIP(req)
		}
		db := ipgeo.Get()

		info_cn, err := db.FindInfo(host, "CN")
		if err != nil {
			fmt.Fprintf(resp, "IP %s not found in the database.", host)
			return
		}
		info_en, err := db.FindInfo(host, "EN")
		if err != nil {
			fmt.Fprintf(resp, "IP %s not found in the database.", host)
			return
		}
		resp.Header().Set("Content-Type", "text/plain")
		resp.WriteHeader(http.StatusOK)
		server := mapping.Get(host)
		fmt.Fprintln(resp, "您的IP:", host)
		fmt.Fprintln(resp, "数据库中IP所属位置:", utils.BuildLocation(info_cn))
		if info_cn.IspDomain != "" {
			fmt.Fprintln(resp, "数据库中IP所属运营商:", info_cn.IspDomain)
		}
		fmt.Fprintln(resp, "您被分配的CDN节点为:", server)
		fmt.Fprintln(resp)
		fmt.Fprintln(resp, strings.Repeat("=", 72))

		fmt.Fprintln(resp, "Your IP:", host)
		fmt.Fprintln(resp, "Location for your IP according our database:", utils.BuildLocation(info_en))
		if info_en.IspDomain != "" {
			fmt.Fprintln(resp, "ISP for your IP according our database:", info_en.IspDomain)
		}
		fmt.Fprintln(resp, "Allocated CDN node for you:", server)
		fmt.Fprintln(resp)
	})

	http.HandleFunc("/mapping", func(resp http.ResponseWriter, req *http.Request) {
		host := utils.GetRemoteIP(req)
		resp.Header().Set("Content-Type", "text/plain")
		resp.WriteHeader(http.StatusOK)
		server := mapping.Get(host)
		fmt.Fprint(resp, server)
	})
	http.HandleFunc("/getMapping", func(resp http.ResponseWriter, req *http.Request) {
		ip := req.FormValue("ip")
		if net.ParseIP(ip).To4() == nil {
			utils.ResponseWithError(resp, http.StatusBadRequest, errIPv4Only)
			return
		}
		resp.Header().Set("Content-Type", "text/plain")
		resp.WriteHeader(http.StatusOK)
		server := mapping.Get(ip)
		log.Printf("%s => %s\n", ip, server)
		fmt.Fprint(resp, server)
	})
	http.HandleFunc("/getNodes", func(resp http.ResponseWriter, req *http.Request) {
		ip := req.FormValue("ip")
		if net.ParseIP(ip).To4() == nil {
			utils.ResponseWithError(resp, http.StatusBadRequest, errIPv4Only)
			return
		}
		nodes := mapping.GetNodes()
		if nodes == nil {
			utils.ResponseWithJsonError(resp, http.StatusInternalServerError, "Unable to get nodes")
			return
		}
		suffix := mapping.GetSuffix(ip)
		resp.Header().Set("Content-Type", "application/json")
		resp.WriteHeader(http.StatusOK)
		jsonEncoder := json.NewEncoder(resp)
		ret := make([]string, 0, len(nodes))
		for _, node := range nodes {
			ret = append(ret, node+suffix)
		}
		jsonEncoder.Encode(ret)
	})
}
